/* opto_robot
 * 
 * Akadálykikerülő robot, amely az akadályokat három TCRT5000
 * szenzorral érzékeli. 
 * 
 * A program ötlete és stratégiája az alábbi cikken alapul:
 * https://robotepites.blog.hu/2018/01/06/akadalykikerulo_robot_v1
 * A stratégián és a technikai megvalósításon kisebb változtatásokat
 * eszközöltünk.
 * 
 * Részletes leírás:
 * http://megtestesules.info/hobbielektronika/2020.html
 * Lásd a 2021. január 28. és 2021. február 11. előadásokat!
 * 
 * Tested with Arduino IDE 1,8,13
 */

#define B1A 7          // Bal motor előre
#define B1B 6          // Bal motor hátra
#define A1A 5          // Jobb motor előre
#define A1B 4          // Jobb motor hátra

#define L_sense A0
#define L_LED   10
#define M_sense A1
#define M_LED    9
#define R_sense A2
#define R_LED    8
#define NMEAS   50

#define L_LIMIT_LOW 50
#define L_LIMIT     100
#define M_LIMIT_LOW 30
#define M_LIMIT     40
#define R_LIMIT_LOW 50
#define R_LIMIT     100

uint32_t meres(int p_sense, int p_LED) {
  uint32_t sum0, sum1;
  digitalWrite(p_LED, LOW);
  sum0 = 0;
  for (int i = 0; i < NMEAS; i++) {
    sum0 += analogRead(p_sense);
  }
  digitalWrite(p_LED, HIGH);
  sum1 = 0;
  for (int i = 0; i < NMEAS; i++) {
    sum1 += analogRead(p_sense);
  }
  if (sum0 > sum1) sum0 = sum1;
  return ((sum1 - sum0) / NMEAS);
}

void setup() {
  analogReference(DEFAULT);
  pinMode(L_LED, OUTPUT);
  pinMode(M_LED, OUTPUT);
  pinMode(R_LED, OUTPUT);
  // Motorvezérlő lábak konfigurálása
  pinMode(B1A, OUTPUT);
  pinMode(B1B, OUTPUT);
  pinMode(A1A, OUTPUT);
  pinMode(A1B, OUTPUT);
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, LOW);
}

void loop() {
  motor_forward();
  uint32_t bal = meres(L_sense, L_LED);
  uint32_t kozep = meres(M_sense, M_LED);
  uint32_t jobb = meres(R_sense, R_LED);
  if (kozep > M_LIMIT) {
    if ( bal < jobb ) {
      motor_left();
      while ((jobb > R_LIMIT_LOW) || (kozep > M_LIMIT_LOW)) {
        kozep = meres(M_sense, M_LED);
        jobb = meres(R_sense, R_LED);
        delay(50);
      }
    } else {
      motor_right();
      while ((bal > L_LIMIT_LOW) || (kozep > M_LIMIT_LOW)) {
        kozep = meres(M_sense, M_LED);
        bal = meres(L_sense, L_LED);
        delay(50);
      }
    }
  }
  else if (jobb > R_LIMIT) {
    motor_left();
    while (jobb > R_LIMIT_LOW) {
      jobb = meres(R_sense, R_LED);
      delay(50);
    }
  }
  else if (bal > L_LIMIT) {
    motor_right();
    while (bal > L_LIMIT_LOW) {
      bal = meres(L_sense, L_LED);
      delay(50);
    }
  }
}

void motor_forward() {
  digitalWrite(B1A, HIGH);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, HIGH);
  digitalWrite(A1B, LOW);
}

void motor_reverse() {
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, HIGH);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, HIGH);
}

void motor_left() {
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, HIGH);
  digitalWrite(A1A, HIGH);
  digitalWrite(A1B, LOW);
}

void motor_right() {
  digitalWrite(B1A, HIGH);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, HIGH);
}

void motor_stop() {
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, LOW);
}
